@@ -1,17 +1,17 @@
-subroutine kf(yt, ymiss, ydimt, yna, tvh, tvz, tvhz, timevar, zt, tt, rt, ht, qt, a1, p1, &
+subroutine kf(yt, ymiss, ydimt, yna, tvh, tvhz, timevar, zt, tt, rt, ht, qt, a1, p1, &
      at, pt, vtuni, ftuni, ktuni, pinf, pstar, finfuni, fstaruni, kinfuni, kstaruni, d, j, &
      p, m, r, n, lik, optcal, info, vt, ft, kt, lt, finf, fstar, kinf, kstar, linf, lstar, eps)
 
 implicit none
 
-integer, intent(in) ::  p, m, r, n, yna, tvh, tvz, tvhz
+integer, intent(in) ::  p, m, r, n, yna, tvh, tvhz
 integer, intent(inout) :: d, j,info
-integer ::  t, i,k,l
+integer ::  t, i,k
 integer, intent(inout),dimension(n) :: ydimt
 integer, intent(in), dimension(5) :: timevar
 double precision, intent(inout), dimension(p,n) :: yt
 integer, intent(in), dimension(p,n) :: ymiss
-double precision, intent(inout), dimension(p,m,(n-1)*tvz+1) :: zt  
+double precision, intent(inout), dimension(p,m,(n-1)*tvhz+1) :: zt  
 double precision, intent(in), dimension(m,m,(n-1)*timevar(1)+1) :: tt 
 double precision, intent(in), dimension(m,r,(n-1)*timevar(2)+1) :: rt 
 double precision, intent(inout), dimension(p,p,(n-1)*timevar(4)+1) :: ht 
@@ -19,86 +19,64 @@ double precision, intent(in), dimension(r,r,(n-1)*timevar(3)+1) :: qt
 double precision, intent(in), dimension(m) :: a1
 double precision, intent(in), dimension(m,m) ::  p1
 double precision, intent(inout), dimension(m,n+1) :: at
-double precision, intent(inout), dimension(m,m,n+1) :: pt
-double precision, intent(inout), dimension(p,n) :: vtuni
-double precision, intent(inout), dimension(p,n) :: vt
-double precision, intent(inout), dimension(p,n) :: ftuni
-double precision, intent(inout), dimension(m,p,n) :: ktuni
-double precision, intent(inout),dimension(m,m,n+1) ::  pstar
-double precision, intent(inout),dimension(m,m,n+1) ::  pinf
-double precision, intent(inout),dimension(m,p,n) ::  kstaruni
-double precision, intent(inout),dimension(m,p,n) ::  kinfuni
-double precision, intent(inout),dimension(p,n) ::  fstaruni
-double precision, intent(inout),dimension(p,n) ::  finfuni
-double precision, intent(inout), dimension(p,p,n) :: ft
-double precision, intent(inout), dimension(m,p,n) :: kt
-double precision, intent(inout), dimension(m,m,n) :: lt
-double precision, intent(inout),dimension(m,p,n) ::  kstar
-double precision, intent(inout),dimension(m,p,n) ::  kinf
-double precision, intent(inout),dimension(p,p,n) ::  fstar
-double precision, intent(inout),dimension(p,p,n) ::  finf
-double precision, intent(inout), dimension(m,m,n) :: linf
-double precision, intent(inout), dimension(m,m,n) :: lstar
+double precision, intent(inout), dimension(m,m,n+1) :: pt, pstar, pinf
+double precision, intent(inout), dimension(p,n) :: vtuni, vt, ftuni, fstaruni,finfuni
+double precision, intent(inout), dimension(m,p,n) :: ktuni, kstaruni, kinfuni, kt, kstar, kinf
+double precision, intent(inout), dimension(p,p,n) :: ft, fstar, finf
+double precision, intent(inout), dimension(m,m,n) :: lt, linf, lstar
 integer, intent(in), dimension(4) :: optcal 
 double precision, dimension(p,m) :: pm
 double precision, dimension(m,p) ::  mp
 double precision, dimension(p,p) ::  cholft
 double precision, intent(inout) :: lik
-double precision, dimension(m) :: arec
-double precision, dimension(m,m) :: prec
-double precision, dimension(m,m) ::  psrec
-double precision, dimension(m,m) ::  pirec
+double precision, dimension(m) :: arec, m1
+double precision, dimension(m,m) :: prec, psrec, pirec
 double precision, dimension(m,r) :: mr
-double precision, dimension(m,m) ::  im
-double precision, dimension(m) ::  m1
-double precision, dimension(m,m) ::  mm
+double precision, dimension(m,m) ::  im, mm
 integer, dimension((n-1)*timevar(4)+1) :: hdiagtest
 double precision, intent(in) :: eps
 double precision, dimension(p,n) :: yy
 double precision, dimension(p,m,(n-1)*timevar(5)+1) :: zz
 double precision, dimension(p,p,(n-1)*timevar(4)+1) :: hh
-
-double precision, dimension(p) :: diaghelp
 double precision, dimension(p,p,(n-1)*tvh+1) :: ldl 
 double precision, dimension(p,(n-1)*tvh+1) :: diag
 integer, dimension(p) :: vp
 
 
-external dcopy
-external dgemm
-external dgemv
-external daxpy
-external dsyr
-external dger
-external dposv
-external dtrsm
-external dtrsv
-external dsymm
-external dsymv
-external dsyevr
-external dpotrf
+external dcopy, dgemm, dgemv, daxpy, dsyr, dger, dposv, dtrsm, dtrsv, dsymm, dsymv,dsyevr,dpotrf
+!external dgemm
+!external dgemv
+!external daxpy
+!external dsyr
+!external dger
+!external dposv
+!external dtrsm
+!external dtrsv
+!external dsymm
+!external dsymv
+!external dsyevr
+!external dpotrf
 
 double precision, external :: ddot
 
-do t=1, (n-1)*tvh+1
-   ldl(1:p,1:p,t) = ht(1:p,1:p,(t-1)*timevar(4)+1)
-end do
-
-if(optcal(1)==1) then
-   do t = 1, n
-      yy(1:p,t) = yt(1:p,t)
-   end do
-end if
 
 if(sum(optcal)>0) then
+   
    do t = 1, (n-1)*timevar(5)+1
       zz(1:p,1:m,t) = zt(1:p,1:m,t)
    end do
-end if
-if(optcal(2)==1) then
-   do t = 1, (n-1)*timevar(4)+1
-      hh(1:p,1:p,t) = ht(1:p,1:p,t)
-   end do
+   
+   if(optcal(1)==1) then
+      do t = 1, n
+         yy(1:p,t) = yt(1:p,t)
+      end do
+   end if
+   
+   if(optcal(2)==1) then
+      do t = 1, (n-1)*timevar(4)+1
+         hh(1:p,1:p,t) = ht(1:p,1:p,t)
+      end do
+   end if
 end if
 
 do t = 1, n
@@ -109,6 +87,14 @@ do i=1,p
    vp(i)=i
 end do
 
+do t=1, (n-1)*tvh+1
+   ldl(1:p,1:p,t) = ht(1:p,1:p,(t-1)*timevar(4)+1)
+   do i = 1, p
+      diag(i,t) = ldl(i,i,t)
+   end do
+end do
+
+
 hdiagtest=0
 
 if(p>1) then
@@ -124,33 +110,17 @@ if(p>1) then
    end do 
    if(sum(hdiagtest)>0) then
       do t=1, ((n-1)*timevar(4)+1)
-         !ldl decomposition
-         if(hdiagtest(t)==1) then
-            do l=1,p
-               do i=l,p
-                  if(i==l .AND. i>1) then
-                     do k=1, i-1
-                        diaghelp(k) = ldl(k,k,t)
-                     end do
-                     ldl(i,i,t) = ldl(i,i,t) - ddot(i-1,ldl(i,1:(i-1),t)**2,1,diaghelp(1:i-1),1)            
-                  end if
-                  if(i>l) then
-                     if(l>1) then
-                        ldl(i,l,t) = (ldl(i,l,t) - &
-                             ddot(l-1,ldl(i,1:(l-1),t),1,matmul(ldl(l,1:(l-1),t),ldl(1:(l-1),1:(l-1),t)),1))/ldl(l,l,t)
-                     else
-                        ldl(i,l,t) = ldl(i,l,t)/ldl(l,l,t)
-                     end if
-                  end if
-               end do
-            end do
+         call dpotrf('l',p,ldl(1:p,1:p,t),p,info)
+         if(info /=0) then
+            info=1
+            return
          end if
-      end do
-      if(timevar(4)==1 .AND. timevar(5)==0) then !ht riippuu ajasta zt ei
-         do t=2,n
-            zt(1:p,1:m,t) = zt(1:p,1:m,1)
+         do i = 1, p
+            diag(i,t) = ldl(i,i,t)
+            ldl(1:p,i,t) = ldl(1:p,i,t)/diag(i,t)
          end do
-      end if
+         diag(1:p,t) = diag(1:p,t)**2
+      end do
       do t=1,(n-1)*tvhz+1
          call dtrsm('l','l','n','u',p,m,1.0d0,ldl(1:p,1:p,(t-1)*timevar(4)+1),p,zt(1:p,1:m,t),p) !solve z*=inv(L) * zt
       end do
@@ -167,7 +137,6 @@ if(p>1) then
          end do
       end if
       do t = 1, n
-!         ydimt(t) = sum(ymiss(1:p, t))
          if ((ydimt(t) .NE. p) .AND.( ydimt(t) .NE. 0)) then
             yt(1:ydimt(t), t) = yt((ymiss(1:p,t)*vp), t)
             ldl(1:ydimt(t), 1:ydimt(t),t) = ldl(ymiss(1:p,t)*vp, ymiss(1:p,t)*vp, t)
@@ -179,22 +148,19 @@ if(p>1) then
       end do
    else
       if(sum(hdiagtest)>0) then
-         do t = 1, n
-            if(hdiagtest((t-1)*timevar(4)+1)==1) then
-               call dtrsv('l','n','u',p,ldl(1:p,1:p,(t-1)*timevar(4)+1),p,yt(1:p,t),1) !solve y*=inv(L) * yt, 
-            end if
-         end do
+         if(timevar(4)==0) then
+            call dtrsm('l','l','n','u',p,n,1.0d0,ldl(1:p,1:p,1),p,yt(1:p,1:n),p) !solve y*=inv(L) * yt
+         else
+            do t = 1, n
+               if(hdiagtest((t-1)*timevar(4)+1)==1) then
+                  call dtrsv('l','n','u',p,ldl(1:p,1:p,(t-1)*timevar(4)+1),p,yt(1:p,t),1) !solve y*=inv(L) * yt, 
+               end if
+            end do
+         end if
       end if
    end if
 end if
 
-do t=1, (n-1)*tvh+1
-   do i = 1, p
-      diag(i,t) = ldl(i,i,t)
-   end do
-end do
-
-
 lik = 0.0d0
 
 im = 0.0d0
@@ -213,13 +179,13 @@ if(maxval(abs(pinf(1:m,1:m,1))) > eps) then
    diffuse: do while(d < n)
       d = d+1
       do j=1, ydimt(d)
-         call dsymv('u',m,1.0d0,psrec,m,zt(j,1:m,(d-1)*tvz+1),1,0.0d0,m1,1) 
-         fstaruni(j,d) = ddot(m,zt(j,1:m,(d-1)*tvz+1),1,m1,1)  + diag(j,(d-1)*tvh+1)
-         call dsymv('u',m,1.0d0,pirec,m,zt(j,1:m,(d-1)*tvz+1),1,0.0d0,m1,1)
-         finfuni(j,d) = ddot(m,zt(j,1:m,(d-1)*tvz+1),1,m1,1)! finf
-         vtuni(j,d) = yt(j,d) - ddot(m,zt(j,1:m,(d-1)*tvz+1),1,arec,1) !arec
-         call dsymv('u',m,1.0d0,psrec,m,zt(j,1:m,(d-1)*tvz+1),1,0.0d0,kstaruni(1:m,j,d),1) ! kstar_t,i = pstar_t,i*t(z_t,i)
-         call dsymv('u',m,1.0d0,pirec,m,zt(j,1:m,(d-1)*tvz+1),1,0.0d0,kinfuni(1:m,j,d),1) ! kinf_t,i = pinf_t,i*t(z_t,i)
+         call dsymv('u',m,1.0d0,psrec,m,zt(j,1:m,(d-1)*tvhz+1),1,0.0d0,m1,1) 
+         fstaruni(j,d) = ddot(m,zt(j,1:m,(d-1)*tvhz+1),1,m1,1)  + diag(j,(d-1)*tvh+1)
+         call dsymv('u',m,1.0d0,pirec,m,zt(j,1:m,(d-1)*tvhz+1),1,0.0d0,m1,1)
+         finfuni(j,d) = ddot(m,zt(j,1:m,(d-1)*tvhz+1),1,m1,1)! finf
+         vtuni(j,d) = yt(j,d) - ddot(m,zt(j,1:m,(d-1)*tvhz+1),1,arec,1) !arec
+         call dsymv('u',m,1.0d0,psrec,m,zt(j,1:m,(d-1)*tvhz+1),1,0.0d0,kstaruni(1:m,j,d),1) ! kstar_t,i = pstar_t,i*t(z_t,i)
+         call dsymv('u',m,1.0d0,pirec,m,zt(j,1:m,(d-1)*tvhz+1),1,0.0d0,kinfuni(1:m,j,d),1) ! kinf_t,i = pinf_t,i*t(z_t,i)
          if (abs(finfuni(j,d)) > eps) then
             call daxpy(m,vtuni(j,d)/finfuni(j,d),kinfuni(1:m,j,d),1,arec,1) !a_rec = a_rec + kinf(:,i,t)*vt(:,t)/finf(j,d)
             call dsyr('u',m,fstaruni(j,d)/(finfuni(j,d)**2),kinfuni(1:m,j,d),1,psrec,m) !psrec = psrec +  kinf*kinf'*fstar/finf^2
@@ -261,9 +227,9 @@ if(maxval(abs(pinf(1:m,1:m,1))) > eps) then
 !non-diffuse filtering begins
    prec = psrec
    do i = j+1, ydimt(d)     
-      vtuni(i,d) = yt(i,d) - ddot(m,zt(i,1:m,(d-1)*tvz+1),1,arec,1) !vtuni         
-      call dsymv('u',m,1.0d0,prec,m,zt(i,1:m,(d-1)*tvz+1),1,0.0d0,m1,1) ! p symmetric!
-      ftuni(i,d) = ddot(m,zt(i,1:m,(d-1)*tvz+1),1,m1,1) + diag(i,(d-1)*tvh+1)
+      vtuni(i,d) = yt(i,d) - ddot(m,zt(i,1:m,(d-1)*tvhz+1),1,arec,1) !vtuni         
+      call dsymv('u',m,1.0d0,prec,m,zt(i,1:m,(d-1)*tvhz+1),1,0.0d0,m1,1) ! p symmetric!
+      ftuni(i,d) = ddot(m,zt(i,1:m,(d-1)*tvhz+1),1,m1,1) + diag(i,(d-1)*tvh+1)
       if (abs(ftuni(i,d)) > eps) then !ftuni/=0
          call daxpy(m,(1.0d0)/ftuni(i,d),m1,1,ktuni(1:m,i,d),1) !ktuni
          call daxpy(m,vtuni(i,d),ktuni(1:m,i,d),1,arec,1) !a_rec = a_rec + ktuni(:,i,t)*vtuni(:,t)
@@ -305,9 +271,9 @@ if(d==0) then
 end if
 do t = d+1, n
    do i = 1, ydimt(t)     
-      vtuni(i,t) = yt(i,t) - ddot(m,zt(i,1:m,(t-1)*tvz+1),1,arec,1) !univariate vt           
-      call dsymv('u',m,1.0d0,prec,m,zt(i,1:m,(t-1)*tvz+1),1,0.0d0,m1,1) ! p symmetric!
-      ftuni(i,t) = ddot(m,zt(i,1:m,(t-1)*tvz+1),1,m1,1)  + diag(i,(t-1)*tvh+1) !ftuni      
+      vtuni(i,t) = yt(i,t) - ddot(m,zt(i,1:m,(t-1)*tvhz+1),1,arec,1) !univariate vt           
+      call dsymv('u',m,1.0d0,prec,m,zt(i,1:m,(t-1)*tvhz+1),1,0.0d0,m1,1) ! p symmetric!
+      ftuni(i,t) = ddot(m,zt(i,1:m,(t-1)*tvhz+1),1,m1,1)  + diag(i,(t-1)*tvh+1) !ftuni      
       if (abs(ftuni(i,t)) > eps) then !ft/=0
          call daxpy(m,(1.0d0)/ftuni(i,t),m1,1,ktuni(1:m,i,t),1) !kt kirja
          call daxpy(m,vtuni(i,t),ktuni(1:m,i,t),1,arec,1) !a_rec = a_rec + kt(:,i,t)*vt(:,t) 
@@ -361,7 +327,7 @@ if(sum(optcal)>0) then
                vt(1,t)=vtuni(1,t)
             else
                call dcopy(ydimt(t),yy(1:ydimt(t),t),1,vt(1:ydimt(t),t),1)
-               call dgemv('n',ydimt(t),m,-1.0d0,zz(1:ydimt(t),1:m,(t-1)*tvz+1),ydimt(t),at(1:m,t),1,1.0d0,vt(1:ydimt(t),t),1)
+               call dgemv('n',ydimt(t),m,-1.0d0,zz(1:ydimt(t),1:m,(t-1)*tvhz+1),ydimt(t),at(1:m,t),1,1.0d0,vt(1:ydimt(t),t),1)
             end if
          end if
       end do
@@ -375,14 +341,14 @@ if(sum(optcal)>0) then
                fstar(1,1,t) = fstaruni(1,t)
                finf(1,1,t) = finfuni(1,t)
             else
-               call dgemm('n','t',m,ydimt(t),m,1.0d0,pinf(1:m,1:m,t),m,zz(1:ydimt(t),1:m,(t-1)*tvz+1),ydimt(t),&
+               call dgemm('n','t',m,ydimt(t),m,1.0d0,pinf(1:m,1:m,t),m,zz(1:ydimt(t),1:m,(t-1)*tvhz+1),ydimt(t),&
                     0.0d0,mp(1:m,1:ydimt(t)),m) !mp = pinf*z'
-               call dgemm('n','n',p,p,m,1.0d0,zz(1:ydimt(t),1:m,(t-1)*tvz+1),p,&
+               call dgemm('n','n',p,p,m,1.0d0,zz(1:ydimt(t),1:m,(t-1)*tvhz+1),p,&
                     mp(1:m,1:ydimt(t)),m,0.0d0,finf(1:ydimt(t),1:ydimt(t),t),ydimt(t)) !finf = z*mp
                
-               call dgemm('n','t',m,ydimt(t),m,1.0d0,pstar(1:m,1:m,t),m,zz(1:ydimt(t),1:m,(t-1)*tvz+1),&
+               call dgemm('n','t',m,ydimt(t),m,1.0d0,pstar(1:m,1:m,t),m,zz(1:ydimt(t),1:m,(t-1)*tvhz+1),&
                     ydimt(t),0.0d0,mp(1:m,1:ydimt(t)),m) !mp = pstar*z'
-               call dgemm('n','n',ydimt(t),ydimt(t),m,1.0d0,zz(1:ydimt(t),1:m,(t-1)*tvz+1),ydimt(t),&
+               call dgemm('n','n',ydimt(t),ydimt(t),m,1.0d0,zz(1:ydimt(t),1:m,(t-1)*tvhz+1),ydimt(t),&
                     mp(1:m,1:ydimt(t)),m,0.0d0,fstar(1:ydimt(t),1:ydimt(t),t),ydimt(t)) !fstar = z*mp 
                fstar(1:ydimt(t),1:ydimt(t),t) = fstar(1:ydimt(t),1:ydimt(t),t) +  hh(1:ydimt(t),1:ydimt(t),(t-1)*tvh+1)
             end if
@@ -394,10 +360,10 @@ if(sum(optcal)>0) then
                ft(1,1,t) = ftuni(1,t)
             else
                ft(1:ydimt(t),1:ydimt(t),t) =  hh(1:ydimt(t),1:ydimt(t),(t-1)*tvh+1)
-               call dsymm('r','u',ydimt(t),m,1.0d0,pt(1:m,1:m,t),m,zz(1:ydimt(t),1:m,(t-1)*tvz+1),ydimt(t),&
+               call dsymm('r','u',ydimt(t),m,1.0d0,pt(1:m,1:m,t),m,zz(1:ydimt(t),1:m,(t-1)*tvhz+1),ydimt(t),&
                     0.0d0,pm(1:ydimt(t),1:m),ydimt(t))
                call dgemm('n','t',ydimt(t),ydimt(t),m,1.0d0,pm(1:ydimt(t),1:m),ydimt(t),&
-                    zz(1:ydimt(t),1:m,(t-1)*tvz+1),ydimt(t),1.0d0,ft(1:ydimt(t),1:ydimt(t),t),ydimt(t)) !ft
+                    zz(1:ydimt(t),1:m,(t-1)*tvhz+1),ydimt(t),1.0d0,ft(1:ydimt(t),1:ydimt(t),t),ydimt(t)) !ft
             end if
          end if
       end do
@@ -407,7 +373,7 @@ if(sum(optcal)>0) then
       mulkd: do t=1,d
          if(ydimt(t)>0) then
             if(maxval(abs(finf(1:ydimt(t),1:ydimt(t),t)))<eps) then !finf=0
-               call dgemm('n','t',m,ydimt(t),m,1.0d0,pstar(1:m,1:m,t),m,zz(1:ydimt(t),1:m,(t-1)*tvz+1),p,0.0d0,&
+               call dgemm('n','t',m,ydimt(t),m,1.0d0,pstar(1:m,1:m,t),m,zz(1:ydimt(t),1:m,(t-1)*tvhz+1),p,0.0d0,&
                     mp(1:m,1:ydimt(t)),m) !mp = pstar*z'
                call dgemm('n','n',m,ydimt(t),m,1.0d0,tt(1:m,1:m,(t-1)*timevar(1)+1),m,mp(1:m,1:ydimt(t)),m,0.0d0,&
                     kstar(1:m,1:ydimt(t),t),m) !kstar = t*mp
@@ -420,7 +386,7 @@ if(sum(optcal)>0) then
                end if
                kstar(1:m,1:ydimt(t),t) = transpose(pm(1:ydimt(t),1:m))
             else
-               pm(1:ydimt(t),1:m) = zz(1:ydimt(t),1:m,(t-1)*tvz+1)
+               pm(1:ydimt(t),1:m) = zz(1:ydimt(t),1:m,(t-1)*tvhz+1)
                cholft(1:ydimt(t),1:ydimt(t)) = finf(1:ydimt(t),1:ydimt(t),t)
                call dposv('l',ydimt(t),m,cholft(1:ydimt(t),1:ydimt(t)),ydimt(t),pm(1:ydimt(t),1:m),ydimt(t),info)
                if(info/=0) then !if finf not pos.def
@@ -434,7 +400,7 @@ if(sum(optcal)>0) then
                call dgemm('n','n',m,ydimt(t),m,1.0d0,tt(1:m,1:m,(t-1)*timevar(1)+1),m,mp(1:m,1:ydimt(t)),&
                     m,0.0d0,kinf(1:m,1:ydimt(t),t),m) !kinf = t*pinf*z'*inv(finf)
                
-               call dgemm('n','t',m,ydimt(t),m,1.0d0,pstar(1:m,1:m,t),m,zz(1:ydimt(t),1:m,(t-1)*tvz+1),ydimt(t),0.0d0,&
+               call dgemm('n','t',m,ydimt(t),m,1.0d0,pstar(1:m,1:m,t),m,zz(1:ydimt(t),1:m,(t-1)*tvhz+1),ydimt(t),0.0d0,&
                     mp(1:m,1:ydimt(t)),m) !mp = pstar*z'    
                call dgemm('n','n',m,ydimt(t),m,1.0d0,tt(1:m,1:m,(t-1)*timevar(1)+1),m,mp(1:m,1:ydimt(t)),m,&
                     0.0d0,kstar(1:m,1:ydimt(t),t),m) !kstar = t*mp
@@ -459,7 +425,8 @@ if(sum(optcal)>0) then
                   call dgemv('n',m,m,1.0d0,tt(1:m,1:m,(t-1)*timevar(1)+1),m,ktuni(1:m,1,t),1,0.0d0,kt(1:m,1,t),1) !kt=t*ktuni
                else
                   call dsymm('l','u',m,m,1.0d0,pt(1:m,1:m,t),m,tt(1:m,1:m,(t-1)*timevar(1)+1),m,0.0d0,mm,m) !TP
-                  call dgemm('n','t',m,ydimt(t),m,1.0d0,mm,m,zz(1:ydimt(t),1:m,(t-1)*tvz+1),ydimt(t),1.0d0,kt(1:m,1:ydimt(t),t),m) !TPZ'
+                  call dgemm('n','t',m,ydimt(t),m,1.0d0,mm,m,zz(1:ydimt(t),1:m,(t-1)*tvhz+1),ydimt(t),1.0d0, &
+                       kt(1:m,1:ydimt(t),t),m) !TPZ'
                   cholft(1:ydimt(t),1:ydimt(t)) = transpose(ft(1:ydimt(t),1:ydimt(t),t))
                   pm(1:ydimt(t),1:m) = transpose(kt(1:m,1:ydimt(t),t))
                   call dposv('l',ydimt(t),m,cholft(1:ydimt(t),1:ydimt(t)),ydimt(t),pm(1:ydimt(t),1:m),ydimt(t),info)
@@ -479,13 +446,13 @@ if(sum(optcal)>0) then
             if(maxval(abs(finf(1:ydimt(t),1:ydimt(t),t)))<eps) then !finf=0
                lstar(1:m,1:m,t) = tt(1:m,1:m,(t-1)*timevar(1)+1)
                call dgemm('n','n',m,m,ydimt(t),-1.0d0,kstar(1:m,1:ydimt(t),t),m,&
-                    zz(1:ydimt(t),1:m,(t-1)*tvz+1),ydimt(t),1.0d0,lstar(1:m,1:m,t),m) !lstar
+                    zz(1:ydimt(t),1:m,(t-1)*tvhz+1),ydimt(t),1.0d0,lstar(1:m,1:m,t),m) !lstar
             else            
                linf(1:m,1:m,t) = tt(1:m,1:m,(t-1)*timevar(1)+1)
                call dgemm('n','n',m,m,ydimt(t),-1.0d0,kinf(1:m,1:ydimt(t),t),m,zz(1:ydimt(t),&
-                 1:m,(t-1)*tvz+1),ydimt(t),1.0d0,linf(1:m,1:m,t),m) !linf 
+                 1:m,(t-1)*tvhz+1),ydimt(t),1.0d0,linf(1:m,1:m,t),m) !linf 
                call dgemm('n','n',m,m,ydimt(t),-1.0d0,kstar(1:m,1:ydimt(t),t),m,zz(1:ydimt(t),&
-                    1:m,(t-1)*tvz+1),ydimt(t),0.0d0,lstar(1:m,1:m,t),m) !lstar
+                    1:m,(t-1)*tvhz+1),ydimt(t),0.0d0,lstar(1:m,1:m,t),m) !lstar
             end if
          end if
       end do
@@ -493,7 +460,7 @@ if(sum(optcal)>0) then
          lt(1:m,1:m,t) = tt(1:m,1:m,(t-1)*timevar(1)+1)
          if(ydimt(t)>0) then
             call dgemm('n','n',m,m,ydimt(t),-1.0d0,kt(1:m,1:ydimt(t),t),m,zz(1:ydimt(t),1:m,&
-                 (t-1)*tvz+1),ydimt(t),1.0d0,lt(1:m,1:m,t),m) !lt = t - kz
+                 (t-1)*tvhz+1),ydimt(t),1.0d0,lt(1:m,1:m,t),m) !lt = t - kz
          end if
       end do
    end if
@@ -505,14 +472,14 @@ if(sum(optcal)>0) then
             fstar(1,1,t) = fstaruni(1,t)
             finf(1,1,t) = finfuni(1,t)
          else
-            call dgemm('n','t',m,p,m,1.0d0,pinf(1:m,1:m,t),m,zt(1:p,1:m,(t-1)*tvz+1),p,&
+            call dgemm('n','t',m,p,m,1.0d0,pinf(1:m,1:m,t),m,zt(1:p,1:m,(t-1)*tvhz+1),p,&
                  0.0d0,mp(1:m,1:p),m) !mp = pinf*z'
-            call dgemm('n','n',p,p,m,1.0d0,zt(1:p,1:m,(t-1)*tvz+1),p,&
+            call dgemm('n','n',p,p,m,1.0d0,zt(1:p,1:m,(t-1)*tvhz+1),p,&
                  mp(1:m,1:p),m,0.0d0,finf(1:p,1:p,t),p) !finf = z*mp
             
-            call dgemm('n','t',m,p,m,1.0d0,pstar(1:m,1:m,t),m,zt(1:p,1:m,(t-1)*tvz+1),&
+            call dgemm('n','t',m,p,m,1.0d0,pstar(1:m,1:m,t),m,zt(1:p,1:m,(t-1)*tvhz+1),&
                  p,0.0d0,mp(1:m,1:p),m) !mp = pstar*z'
-            call dgemm('n','n',p,p,m,1.0d0,zt(1:p,1:m,(t-1)*tvz+1),p,&
+            call dgemm('n','n',p,p,m,1.0d0,zt(1:p,1:m,(t-1)*tvhz+1),p,&
                  mp(1:m,1:p),m,0.0d0,fstar(1:p,1:p,t),p) !fstar = z*mp 
             fstar(1:p,1:p,t) = fstar(1:p,1:p,t) +  ht(1:p,1:p,(t-1)*tvh+1)
          end if
@@ -522,14 +489,17 @@ if(sum(optcal)>0) then
             ft(1,1,t) = ftuni(1,t)
          else
             ft(1:p,1:p,t) =  ht(1:p,1:p,(t-1)*tvh+1)
-            call dsymm('r','u',p,m,1.0d0,pt(1:m,1:m,t),m,zt(1:p,1:m,(t-1)*tvz+1),p,&
+            call dsymm('r','u',p,m,1.0d0,pt(1:m,1:m,t),m,zt(1:p,1:m,(t-1)*tvhz+1),p,&
                  0.0d0,pm(1:p,1:m),p)
             call dgemm('n','t',p,p,m,1.0d0,pm(1:p,1:m),p,&
-                 zt(1:p,1:m,(t-1)*tvz+1),p,1.0d0,ft(1:p,1:p,t),p) !ft
+                 zt(1:p,1:m,(t-1)*tvhz+1),p,1.0d0,ft(1:p,1:p,t),p) !ft
          end if
       end do
    end if
    
 end if
+
 end subroutine
  
+
+
